@@ -80,21 +80,21 @@ impl Float for f64 {
 ///
 /// Labels are countable, comparable and hashable. Currently null-type (no targets),
 /// boolean (binary task) and usize, strings (multi-label tasks) are supported.
-pub trait Label: PartialEq + Eq + Hash + Clone {}
+pub trait Label: PartialEq + Eq + Hash + Clone + Ord + fmt::Debug + Default {}
 
 impl Label for bool {}
 impl Label for usize {}
 impl Label for String {}
 impl Label for () {}
 impl Label for &str {}
-impl Label for Option<usize> {}
+impl<L: Label> Label for Option<L> {}
 
 /// Probability types
 ///
 /// This helper struct exists to distinguish probabilities from floating points. For example SVM
 /// selects regression or classification training, based on the target type, and could not
 /// distinguish them without a new-type definition.
-#[derive(Debug, Copy, Clone)]
+#[derive(Debug, Copy, Clone, Default)]
 pub struct Pr(pub f32);
 
 impl Pr {
@@ -524,7 +524,7 @@ mod tests {
         );
     }
 
-    use crate::traits::{Fit, PredictRef};
+    use crate::traits::{Fit, PredictInplace};
     use ndarray::ArrayView2;
     use thiserror::Error;
 
@@ -561,15 +561,33 @@ mod tests {
         }
     }
 
-    impl<'b> PredictRef<ArrayView2<'b, f64>, Array1<f64>> for MockFittableResult {
-        fn predict_ref<'a>(&'a self, _x: &'a ArrayView2<'b, f64>) -> Array1<f64> {
-            array![0.]
+    impl<'b> PredictInplace<ArrayView2<'b, f64>, Array1<f64>> for MockFittableResult {
+        fn predict_inplace<'a>(&'a self, x: &'a ArrayView2<'b, f64>, y: &mut Array1<f64>) {
+            assert_eq!(
+                x.nrows(),
+                y.len(),
+                "The number of data points must match the number of output targets."
+            );
+            *y = array![0.];
+        }
+
+        fn default_target(&self, x: &ArrayView2<f64>) -> Array1<f64> {
+            Array1::zeros(x.nrows())
         }
     }
 
-    impl<'b> PredictRef<ArrayView2<'b, f64>, Array2<f64>> for MockFittableResult {
-        fn predict_ref<'a>(&'a self, _x: &'a ArrayView2<'b, f64>) -> Array2<f64> {
-            array![[0., 0.]]
+    impl<'b> PredictInplace<ArrayView2<'b, f64>, Array2<f64>> for MockFittableResult {
+        fn predict_inplace<'a>(&'a self, x: &'a ArrayView2<'b, f64>, y: &mut Array2<f64>) {
+            assert_eq!(
+                y.shape(),
+                &[x.nrows(), 2],
+                "The number of data points must match the number of output targets."
+            );
+            *y = array![[0., 0.]];
+        }
+
+        fn default_target(&self, x: &ArrayView2<f64>) -> Array2<f64> {
+            Array2::zeros((x.nrows(), 2))
         }
     }
 
@@ -581,9 +599,8 @@ mod tests {
         let mut dataset: Dataset<f64, f64> = (records, targets).into();
         let params = MockFittable { mock_var: 1 };
 
-        for (i, (model, validation_set)) in dataset
-            .iter_fold(5, |v| params.fit(&v).unwrap())
-            .enumerate()
+        for (i, (model, validation_set)) in
+            dataset.iter_fold(5, |v| params.fit(v).unwrap()).enumerate()
         {
             assert_eq!(model.mock_var, 4);
             assert_eq!(validation_set.records().row(0)[0] as usize, i + 1);
@@ -605,9 +622,8 @@ mod tests {
         // If we request three folds from a dataset with 5 samples it will cut the
         // last two samples from the folds and always add them as a tail of the training
         // data
-        for (i, (model, validation_set)) in dataset
-            .iter_fold(3, |v| params.fit(&v).unwrap())
-            .enumerate()
+        for (i, (model, validation_set)) in
+            dataset.iter_fold(3, |v| params.fit(v).unwrap()).enumerate()
         {
             assert_eq!(model.mock_var, 4);
             assert_eq!(validation_set.records().row(0)[0] as usize, i + 1);
@@ -619,9 +635,8 @@ mod tests {
         }
 
         // the same goes for the last sample if we choose 4 folds
-        for (i, (model, validation_set)) in dataset
-            .iter_fold(4, |v| params.fit(&v).unwrap())
-            .enumerate()
+        for (i, (model, validation_set)) in
+            dataset.iter_fold(4, |v| params.fit(v).unwrap()).enumerate()
         {
             assert_eq!(model.mock_var, 4);
             assert_eq!(validation_set.records().row(0)[0] as usize, i + 1);
@@ -634,9 +649,8 @@ mod tests {
 
         // if we choose 2 folds then again the last sample will be only
         // used for trainig
-        for (i, (model, validation_set)) in dataset
-            .iter_fold(2, |v| params.fit(&v).unwrap())
-            .enumerate()
+        for (i, (model, validation_set)) in
+            dataset.iter_fold(2, |v| params.fit(v).unwrap()).enumerate()
         {
             assert_eq!(model.mock_var, 3);
             assert_eq!(validation_set.targets().dim(), (2, 1));
@@ -652,7 +666,7 @@ mod tests {
         let targets = Array1::from_shape_vec(5, vec![1., 2., 3., 4., 5.]).unwrap();
         let mut dataset: Dataset<f64, f64> = (records, targets).into();
         let params = MockFittable { mock_var: 1 };
-        let _ = dataset.iter_fold(0, |v| params.fit(&v)).enumerate();
+        let _ = dataset.iter_fold(0, |v| params.fit(v)).enumerate();
     }
 
     #[test]
@@ -663,7 +677,7 @@ mod tests {
         let targets = Array1::from_shape_vec(5, vec![1., 2., 3., 4., 5.]).unwrap();
         let mut dataset: Dataset<f64, f64> = (records, targets).into();
         let params = MockFittable { mock_var: 1 };
-        let _ = dataset.iter_fold(6, |v| params.fit(&v)).enumerate();
+        let _ = dataset.iter_fold(6, |v| params.fit(v)).enumerate();
     }
 
     #[test]
