@@ -5,7 +5,7 @@
 //!
 
 #[doc(no_inline)]
-pub use crate::error::{Error, Result};
+pub use crate::error::Error;
 
 #[doc(no_inline)]
 pub use crate::traits::*;
@@ -24,3 +24,6 @@ pub use crate::metrics_clustering::SilhouetteScore;
 
 #[doc(no_inline)]
 pub use crate::correlation::PearsonCorrelation;
+
+#[doc(no_inline)]
+pub use crate::param_guard::ParamGuard;
